

#include <PinChangeInt.h> // this library enables the arduino to read the signal from the transmitter
#include <Servo.h>        //  htis library enables the arduino to control the servo



//Assigning channel in pins
#define HYDRO_IN_PIN 5     //input from rudder channel of transmitter inot the arduino pin 5
#define BALLAST_IN_PIN 3   //input from Ballast control of transmitter into the arduino pin 3 
                           // In this sketch the ballast is only used as a switch for retract code


Servo servoHydro;          // output to hydro servo from arduino
Servo servoBallast;        // output to ballast servo from arduino
Servo servoPlaneretract;   // output to Planeretract servo ballast from arduino

int pos = 1100;            //  set the starting point for the servo in microseconds
int retraction = 0;        //  int to start the retraction, only when wanted.  Goes to '1' when triggered
int extendpos = 1800;      //  set the upper point for the servo in microseconds
int retractionpos = 1100;  //  set the lower point for the servo in microseconds.  Same as start for pos
int evenpos = 1500;        //  set the flat or level point for hydroplane

#define HYDRO_FLAG 1      // controls to manage the servo instructions
#define BALLAST_FLAG 2    // controls to manage the servo instructions

volatile uint8_t bUpdateFlagsShared; // controls to manage the servo instructions.  not duplicated
volatile uint16_t unHydroInShared;   // controls to manage the servo instructions
volatile uint16_t unBallastInShared; // note it is duplicated, as below
uint32_t ulHydroStart;  
uint32_t ulBallastStart;


#define HYDRO_OUT_PIN 13           // Assigning channel out pin for connection  to servo
//#define BALLAST_OUT_PIN 7       // Assigning channel out pin for connection  to servo
#define RETRACT_SERVO_PIN A2       // Assigning channel out pin for connection  retract servo
#define SERVO_POWER_PIN A1       // Assigning channel out pin for connection  to relay for hydro
#define POWER_RETRACT_SERVO_PIN A4 // Assigning channel out pin for connection  relayfor retract servo 
 


void setup()
{

  Serial.begin(9600);   //  start serial port
 
  PCintPort::attachInterrupt(HYDRO_IN_PIN, calcHydro, CHANGE);        // reading the receiver
  PCintPort::attachInterrupt(BALLAST_IN_PIN, calcBallast, CHANGE);

  pinMode(SERVO_POWER_PIN, OUTPUT);           
  pinMode(POWER_RETRACT_SERVO_PIN, OUTPUT);

  
  servoHydro.attach(HYDRO_OUT_PIN);               //already there      //  attaching servo instructions to the servos
  //servoBallast.attach(BALLAST_OUT_PIN);
  servoPlaneretract.attach(RETRACT_SERVO_PIN);
  
  //  following routine in setup does a one off set up rouine.  
  
  digitalWrite(SERVO_POWER_PIN, LOW);                   // turns on the hydroplane relay (note for these realys low is on and high is off. can get both sorts
  digitalWrite(POWER_RETRACT_SERVO_PIN, LOW);           // turns on the retract servo
  delay (1000);                                         // gives the servo a moment
  servoPlaneretract.writeMicroseconds(retractionpos);   // sets the retract servo to open at the start of every session.  
  delay (1000);                                         // gives servo a little time to travel
  digitalWrite(POWER_RETRACT_SERVO_PIN, HIGH);          //  turns off the retract servo relay

  
}



void loop()
{
 
  
  static uint16_t unHydroIn;   // servo functions
  static uint16_t unBallastIn;
  static uint8_t bUpdateFlags;  
  
 

  //Serial.println(unHydroIn);    //  to print so oyu can see what is going on
  //Serial.println(unBallastIn);
  //Serial.println(retraction);
  //Serial.println(pos);
  
  
 
   if (unBallastIn < 1200)        //  this is the reading from the ballast channel.  When under 1200 it triggers the process
  {
    retraction = 1;               // trigger for retraction
  } 
  else if (unBallastIn >= 1200)   // opposite
  {
   retraction = 0;                // retraction off
  }


  if ((retraction == 1) && ( extendpos > pos))     //if retract is '1' and the current position is less than the max position 
    {      
     digitalWrite(POWER_RETRACT_SERVO_PIN, LOW);   //  turn on the retract servo relay
     (pos = pos + 4);                              //  let the position increase in increments of 1 'microseconds')
     delay (15);                                   //  speed of increase
     servoPlaneretract.writeMicroseconds(pos);     //  write the changing 'pos' to the servo           
     digitalWrite(POWER_RETRACT_SERVO_PIN, HIGH);  //  keeps the relay off 
    }


 if ((retraction == 0)&& ( retractionpos < pos))   //if retract and position less than 150, then 
    {
    digitalWrite(POWER_RETRACT_SERVO_PIN, LOW);
    (pos = pos - 4);                               //  let the position increase in increments of 1
    delay (15);                                    //  speed of increase
    servoPlaneretract.writeMicroseconds(pos);
    servoPlaneretract.write(pos);                  //  write this to the plane
    digitalWrite(POWER_RETRACT_SERVO_PIN, HIGH);   //  keeps the relay off
  }
  

 if (pos >= (retractionpos+10))                    // If the position of the retract servo, or the pos that drives it, 
                                                   //  is above level, turn the hydro realy off.  May not need the '10'
  {
    digitalWrite(SERVO_POWER_PIN, HIGH);    
  }
  else
  {
    digitalWrite(SERVO_POWER_PIN, LOW);
  }




  
  if (bUpdateFlagsShared)                   
  {
    noInterrupts();    
    bUpdateFlags = bUpdateFlagsShared;

    if (bUpdateFlags & HYDRO_FLAG)
    {
      unHydroIn = unHydroInShared;
    }

    if (bUpdateFlags & BALLAST_FLAG)
    {
      unBallastIn = unBallastInShared;
    }

   


    bUpdateFlagsShared = 0;
    interrupts(); 

  }

  if (bUpdateFlags & HYDRO_FLAG)
  {
    if (servoHydro.readMicroseconds() != unHydroIn)
    {
      servoHydro.writeMicroseconds(unHydroIn);
    }
  }

  if (bUpdateFlags & BALLAST_FLAG)
  {
    if (servoBallast.readMicroseconds() != unBallastIn)
    {
      servoBallast.writeMicroseconds(unBallastIn);
    }
  }

  

  bUpdateFlags = 0;
}

// simple interrupt service routine
void calcHydro()
{
  
  if (digitalRead(HYDRO_IN_PIN) == HIGH)
  {
    ulHydroStart = micros();
  }
  else
  {    
    unHydroInShared = (uint16_t)(micros() - ulHydroStart);    
    bUpdateFlagsShared |= HYDRO_FLAG;
  }
}

void calcBallast()
{
  if (digitalRead(BALLAST_IN_PIN) == HIGH)
  {
    ulBallastStart = micros();
  }
  else
  {
    unBallastInShared = (uint16_t)(micros() - ulBallastStart);
    bUpdateFlagsShared |= BALLAST_FLAG;
  }
}




